#include "Race.h"
#include "Animal.h"
#include <conio.h> // gotoxy() clrscr() getch()
#include <iostream.h> // cout
#include <stdlib.h> // srand()
#include <time.h> // time()
#include <dos.h> // delay()

Race::Race(int xOrg, int yOrg, int maxTracks, int fieldLength):
    m_fieldXOrg(xOrg), m_fieldYOrg(yOrg),
    m_maxTracks(maxTracks),
    m_fieldLength(fieldLength),
    m_racers(new Animal *[maxTracks]),
    m_numberOfRacers(0)
{
    srand((unsigned) time(NULL));
    clrscr();
    drawTracks();
}

Race::~Race()
{
    delete[] m_racers;
}

int Race::numberOfRacers() const
{
    return m_numberOfRacers;
}

BOOL Race::addRacer(Animal &animal)
{
    if (m_numberOfRacers < m_maxTracks)
    {
        m_racers[m_numberOfRacers++] = &animal;
        return TRUE;
    }
    else
        return FALSE;
}

void Race::go() const
{
    gotoxy(1,20);
    cout << "Enter any key to start the race... ";
    getch();
    gotoxy(1,20);
    cout << "                                   ";

    if (m_numberOfRacers <=0)
    {
        gotoxy(1,20);
        cout << "Nobody enjoy the race???" << endl;
        return;
    }

    int i;
    int *runningSequence = new int[m_numberOfRacers];

    moveEachRacerToTheirStartPositions();
    
    while (!endOfRace())
    {
        randomizeSequence(runningSequence);
        for (i=0; i<m_numberOfRacers; i++)
            m_racers[runningSequence[i]]->runOneStep();
        delay(300);
    }
    
    delete[] runningSequence;
}

void Race::moveEachRacerToTheirStartPositions() const
{
    int i;
    for (i=0; i<m_numberOfRacers; i++)
        m_racers[i]->moveTo(m_fieldXOrg, m_fieldYOrg+i);
}

void Race::randomizeSequence(int runningSequence[]) const
{
    int i, total, tmp;
    int *tmpSeq = new int[m_numberOfRacers];

    for (i=0; i<m_numberOfRacers; i++)
        tmpSeq[i] = i;

    total = m_numberOfRacers;
    for (i=0; i<m_numberOfRacers; i++)
    {
        tmp = rand() % total;
        runningSequence[i] = tmpSeq[tmp];
        tmpSeq[tmp] = tmpSeq[total-1];
        total--;
    }
    delete[] tmpSeq;
}

void Race::drawTracks() const
{
    int i;
    gotoxy(m_fieldXOrg-2, m_fieldYOrg-1);
    cout << "+";
    for (i=0; i<m_fieldLength+1; i++)
        cout << "-";
    cout << "+";
    for (i=0; i<m_maxTracks; i++)
    {
        gotoxy(m_fieldXOrg-2, m_fieldYOrg+i);
        cout << "|";
        gotoxy(m_fieldXOrg+m_fieldLength, m_fieldYOrg+i);
        cout << "|";
    }
    gotoxy(m_fieldXOrg-2, m_fieldYOrg+m_maxTracks);
    cout << "+";
    for (i=0; i<m_fieldLength+1; i++)
        cout << "-";
    cout << "+";
}

BOOL Race::endOfRace() const
{
    int i;
    for (i=0; i<m_numberOfRacers; i++)
        if (m_racers[i]->getTo(m_fieldLength+m_fieldXOrg))
            return TRUE;
    return FALSE;
}

void Race::reportResult() const
{
    gotoxy(1, m_fieldYOrg+m_maxTracks+3);
    cout << "The winners are: ";
    int i, j=0;
    for (i=0; i<m_numberOfRacers; i++)
        if (m_racers[i]->getTo(m_fieldLength+m_fieldXOrg))
            m_racers[i]->moveTo(10+3*(j++), 
                                m_fieldYOrg+m_maxTracks+5);

    gotoxy(1,24);
}
